<?php
require_once('autoload.include.php') ;

/**
 * Agrgateur de flux RSS
 */
class AgregateurRSS {
    /**
     * Tableau des lments de flux RSS = tableau d'instances de ElementRSS
     * @var ElementRSS[]
     */
    protected $_elements = array() ;

    /**
     * Ajouter un flux RSS  l'agrgateur
     * @param $url l'URL du flux  ajouter
     *
     * @throws Exception quand le flux ne peut pas tre charg
     * @return void
     */
    public function ajouterFlux($url) {
        // Explorer tous les lments <item> afin de construire un ElementRSS pour chacun d'eux
        $DOMDoc = new DOMDocument() ;
        // Le fichier peut-il tre charg ?
        if (@$DOMDoc->load($url)) {
            // Titre du flux
            $flux = $DOMDoc->getElementsByTagName('channel')->item(0)->getElementsByTagName('title')->item(0)->nodeValue ;
            // Recherche des items
            $items = $DOMDoc->getElementsByTagName('item') ;
            // Parcours des items
            foreach ($items as $i) {
                $this->_elements[] = new ElementRSS($flux, $i) ;
            }
        }
        else {
            throw new Exception("Le flux ".$url."n'a pu tre charg") ;
        }
    }

    /**
     * Production de la liste des lments en HTML
	 *
     * @return string le code HTML
     */
    public function toHTMLRSS() {
        $html = <<<HTML
		
		<table class='rss'>
		  <thead>
			<tr>
			  <th>Date</th>
			  <th>Nom ou Section du site</th>
			  <th class="large">Titre de l'Article</th>
			</tr>
		  </thead>
		  <tfoot>
			<tr>
			  <td colspan="3">------------------ Fin du flux "{$this->_elements[0]->flux()}" ------------------</td>
			</tr>
		  </tfoot>

HTML
;
		$n0=$this->_elements[0]->flux();
        foreach ($this->_elements as $n) {
            if($n0 != $n->flux()) {
				$html .= <<<HTML
		</table>

		<table class='rss'>
		  <thead>
			<tr>
			  <th>Date</th>
			  <th>Nom ou Section du site</th>
			  <th class="large">Titre de l'Article</th>
			</tr>
		  </thead>
		  <tfoot>
			<tr>
			  <td colspan="3">------------------ Fin du flux "{$n->flux()}" ------------------</td>
			</tr>
		  </tfoot>
		  <tbody>
			<tr>
			  <td class='date'>{$n->date()}</td>
			  <td class='flux'>{$n->flux()}</td>
			  <td class='lien'><a class='lien' href='{$n->url()}'>{$n->titre()}</a></td>
			</tr>
		  </tbody>
HTML;
			}else {
				$html .= <<<HTML
		  <tbody>
			<tr>
			  <td class='date'>{$n->date()}</td>
			  <td class='flux'>{$n->flux()}</td>
			  <td class='lien'><a class='lien' href='{$n->url()}'>{$n->titre()}</a></td>
			</tr>
		  </tbody>

HTML;
			}
			$n0=$n->flux();
        }
		$html .= "		</table>";
        return $html ;
    }

	/**
     * Production de la liste des lments en HTML
	 *
     * @return string le code HTML
     */
    public function toHTMLOPML() {
		$html = "";
		foreach($this->_elements as $element)
		{
			$html.=<<<HTML
		<div class='rss'>	
			<span class='date'>{$element->date()}</span>
			<span class='flux'>{$element->flux()}</span> :
			<a class='lien' href={$element->url()}>{$element->titre()}</a>
		</div>
HTML;
		}
        return $html ;
    }
	
    /**
     * Tri des lments par nom de flux source
     *
     * @return void
     */
    public function triFlux() {
        // uasort($this->_elements, 'ElementRSS::compareFlux') ;
        // ou
        uasort($this->_elements, array('ElementRSS', 'compareFlux')) ;
    }

    /**
     * Tri des lments par titre
     *
     * @return void
     */
    public function triTitre() {
        // uasort($this->_elements, 'ElementRSS::compareTitre') ;
        // ou
        uasort($this->_elements, array('ElementRSS', 'compareTitre')) ;
    }

    /**
     * Tri des lments par date
     *
     * @return void
     */
    public function triDate() {
        // uasort($this->_elements, 'ElementRSS::compareDate') ;
        // ou
        uasort($this->_elements, array('ElementRSS', 'compareDate')) ;
    }

	
	/**
     * Production d'un flux RSS
     * @param $title le titre du flux
     * @param $description la description du flux
     * @param $max le nNombre maximal d'lments dans le flux (0 = tous)
     *
     * @return string le code XML du nouveau flux RSS
     */
    public function RSS($title, $description, $max=0) {
        // Rduction du nombre d'lments
        $elements = $max ? array_slice($this->_elements, 0, $max) : $this->_elements ;
        // Dbut du fichier RSS
        $rss = <<<XML
<?xml version="1.0" encoding="UTF-8"?><rss 
	 xmlns:atom="http://www.w3.org/2005/Atom" 
	 xmlns:itunes="http://www.itunes.com/dtds/podcast-1.0.dtd"
     xmlns:dc="http://purl.org/dc/elements/1.1/" 
	 xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"
     xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" 
	 version="2.0">
	<channel>
        <atom:link href="http://{$_SERVER['SERVER_NAME']}/{$_SERVER['PHP_SELF']}" rel="self" type="application/rss+xml" />
        <title>{$title}</title>
        <link>http://{$_SERVER['SERVER_NAME']}/{$_SERVER['PHP_SELF']}</link>
        <description>{$description}</description>
        <language>fr</language>
        <copyright>Copyright William GALBY</copyright>
        <pubDate>Mon, 22 Oct 2018 16:06:04 +0200</pubDate>
        <lastBuildDate>Mon, 22 Oct 2018 16:06:04 +0200</lastBuildDate>
        <ttl>15</ttl>
XML;
        // Parcours des lments
        foreach ($elements as $n) {
            $date = gmdate('D, d M Y H:i:s \G\M\T', $n->timestamp()) ;
            $rss .= <<<XML
            <item>
                <title><![CDATA[{$n->titre()}]]></title>
                <guid><![CDATA[{$n->url()}]]></guid>
                <pubDate>{$date}</pubDate>
                <link><![CDATA[{$n->url()}]]></link>
            </item>
XML;
        }
        // Fin du fichier RSS
        $rss .= <<<XML
    </channel>
</rss>
XML;
        return $rss ;
    }
	
}

