<?php
/**
 *  Elment de flux RSS
 */
class ElementRSS {
    /**
     *  Titre du flux source
     *  @var string
     */
    private $_flux ;
    /**
     *  Titre de l'lment
     *  @var string
     */
    private $_titre ;
    /**
     *  URL associe  l'lment
     *  @var string
     */
    private $_url ;
    /**
     *  Date de publication sous forme de timestamp
     *  @var int
     */
    private $_date ;

    /** Rechercher la valeur d'une balise fille d'un noeud DOM
     * @param $node le noeud DOM  partir duquel la recherche doit tre lance
     * @param $nodeName le nom du noeud cherch
     * @throws Exception quand la balise n'a pu tre trouve
     *
     * @return string la valeur
     */
    public static function valeurElement(DOMElement $node, $nodeName) {
        $list = $node->getElementsByTagName($nodeName) ;
        if ($list->length == 1) {
            return $list->item(0)->firstChild->nodeValue ;
            // return $list->item(0)->nodeValue ; // Fonctionne par "abus de langage" mais ne devrait pas !
        }
        else {
            throw new Exception("La balise '$nodeName' n'a pu tre trouve dans les descendants de '{$node->tagName}'") ;
        }
    }

    /** Constructeur
     * @param $titre_flux le nom du flux source
     * @param $node le noeud DOM source
     */
    public function __construct($titre_flux, DOMelement $node) {
        $this->_flux  = $titre_flux ;
        $this->_titre = self::valeurElement($node, 'title') ;
        $this->_url   = self::valeurElement($node, 'link') ;
        // Certains flux ne comportent pas de date de publication...
        try {
            $this->_date  = strToTime(self::valeurElement($node, 'pubDate')) ;
        }
        catch (Exception $e) {
            $this->_date = time() ;
        }
    }

    /** Accs au nom du flux
     *
     * @return string le nom du flux
     */
    public function flux() {
        return $this->_flux ;
    }

    /** Accs au titre de l'lment
     *
     * @return string le titre de l'lment
     */
    public function titre() {
        return $this->_titre ;
    }

    /** Accs  l'URL de l'lment
     *
     * @return string l'URL de l'lment
     */
    public function url() {
        return $this->_url ;
    }

    /** Accs  la date de publication de l'lment sous forme de timestamp
     *
     * @return int le timestamp de l'lment
     */
    public function timestamp() {
        return $this->_date ;
    }

    /** Accs  la date de publication de l'lment sous forme de texte
     *
     * @return string la date de l'lment sous forme de date
     */
    public function date() {
        return strftime('%d/%m/%Y %H:%M', $this->_date) ;
    }

    /** Comparaison alphabtique des noms des flux de deux lments
     * @param ElementRSS $n1 le premier oprande de la comparaison
     * @param ElementRSS $n2 le second oprande de la comparaison
     *
     * @return int 0, -1 ou 1
     */
    public static function compareFlux(self $n1, self $n2) {
        if ($n1->_flux == $n2->_flux)
            return 0 ;
        return $n1->_flux < $n2->_flux ? -1 : 1 ;
    }

    /** Comparaison alphabtique des titres de deux lments
     * @param ElementRSS $n1 le premier oprande de la comparaison
     * @param ElementRSS $n2 le second oprande de la comparaison
     *
     * @return int 0, -1 ou 1
     */
    public static function compareTitre(self $n1, self $n2) {
        if ($n1->_titre == $n2->_titre)
            return 0 ;
        return $n1->_titre < $n2->_titre ? -1 : 1 ;
    }

    /** Comparaison chronologique inverse des dates de deux lments
     * @param ElementRSS $n1 le premier oprande de la comparaison
     * @param ElementRSS $n2 le second oprande de la comparaison
     *
     * @return int 0, -1 ou 1
     */
    public static function compareDate(self $n1, self $n2) {
        if ($n1->_date == $n2->_date)
            return 0 ;
        return $n1->_date > $n2->_date ? -1 : 1 ;
    }
}

